# -*- coding: utf-8 -*-

from django.db import models


class NameField(models.OneToOneField):
    """
    A name field that has a one-to-one relationship to a `names.Name` instance.
    """

    description = "A full name and its individual components."

    def __init__(self, *args, **kwargs):
        kwargs["to"] = "names.Name"
        super().__init__(*args, **kwargs)



# Generated by Django 2.2.3 on 2019-07-21 18:23

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Name",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("full", models.CharField(max_length=100, verbose_name="Full name")),
                (
                    "title",
                    models.CharField(blank=True, max_length=100, verbose_name="Title"),
                ),
                (
                    "first",
                    models.CharField(
                        blank=True, max_length=100, verbose_name="First name"
                    ),
                ),
                (
                    "last",
                    models.CharField(
                        blank=True, max_length=100, verbose_name="Last name"
                    ),
                ),
                (
                    "middle",
                    models.CharField(
                        blank=True, max_length=100, verbose_name="Middle name"
                    ),
                ),
                (
                    "nickname",
                    models.CharField(
                        blank=True, max_length=100, verbose_name="Nickname"
                    ),
                ),
                (
                    "suffix",
                    models.CharField(blank=True, max_length=100, verbose_name="Suffix"),
                ),
            ],
            options={"abstract": False},
        )
    ]



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import gettext_lazy as _
from nameparser import HumanName
from nameparser.config import CONSTANTS

from .conf import settings


class NameModelMixin(models.Model):
    """
    An abstract base class model mixin that contains a full name
    and its individual components.
    """

    DEFAULTS = {"max_length": settings.MAX_LENGTH, "blank": True}
    full = models.CharField(verbose_name=_("Full name"), max_length=settings.MAX_LENGTH)
    title = models.CharField(verbose_name=_("Title"), **DEFAULTS)
    first = models.CharField(verbose_name=_("First name"), **DEFAULTS)
    last = models.CharField(verbose_name=_("Last name"), **DEFAULTS)
    middle = models.CharField(verbose_name=_("Middle name"), **DEFAULTS)
    nickname = models.CharField(verbose_name=_("Nickname"), **DEFAULTS)
    suffix = models.CharField(verbose_name=_("Suffix"), **DEFAULTS)

    # Stores previous value of `full`
    _initial = None

    class Meta:
        abstract = True

    def __init__(self, *args, **kwargs):
        """
        Stores `self.full` for comparison in `save` method.
        """
        super().__init__(*args, **kwargs)
        self._initial = self.full

    def __str__(self):
        return self.full

    def save(self, *args, **kwargs):
        """
        Parses and saves a full name and its individual components.
        """
        # Use  name components to create a full name. This ensures
        # modifications to individual name components are reflected
        # in the full name.
        name = CONSTANTS.string_format.format(**self.__dict__)

        # If this is a new instance, or `self.full` has been modified,
        # use `self.full` for the full name.
        if not self.pk or self._initial != self.full:
            name = self.full

        instance = self._parse(name=name)
        for attr, val in instance.as_dict().items():
            setattr(self, attr, val)

        self.full = instance.full_name
        super().save(*args, **kwargs)

    @classmethod
    def _parse(cls, name):
        """
        Parses and returns a `HumanName` instance.
        """
        instance = HumanName(
            full_name=name,
            constants=settings.OPTIONS,
            string_format=settings.STRING_FORMAT,
        )

        return instance



# -*- coding: utf-8 -*-

"""
Names uses the `NAMES_SETTINGS` namespace for all settings. For example, your
`settings.py` file may look like this:

NAMES_SETTINGS = {
    "MAX_LENGTH": 175,
    "STRING_FORMAT": "{title} {first} {middle} {last} {suffix} ({nickname})",
    "OPTIONS": {
        "CAPITALIZATION_EXCEPTIONS": (
            ('md', 'M.D.'),
            ('phd', 'Ph.D.'),
        )
    },
}

This module provides a `settings` object that combines all default
`NAMES_SETTINGS` with user-defined `NAMES_SETTINGS`.
"""

from django.conf import settings as dj_settings
from nameparser.config.capitalization import CAPITALIZATION_EXCEPTIONS
from nameparser.config.conjunctions import CONJUNCTIONS
from nameparser.config.prefixes import PREFIXES
from nameparser.config.regexes import REGEXES
from nameparser.config.suffixes import SUFFIX_ACRONYMS, SUFFIX_NOT_ACRONYMS
from nameparser.config.titles import FIRST_NAME_TITLES, TITLES

DEFAULTS = {
    "MAX_LENGTH": 100,
    "STRING_FORMAT": None,
    "EMPTY_ATTRIBUTE_DEFAULT": "",
    "OPTIONS": {
        "TITLES": TITLES,
        "FIRST_NAME_TITLES": FIRST_NAME_TITLES,
        "SUFFIX_ACRONYMS": SUFFIX_ACRONYMS,
        "SUFFIX_NOT_ACRONYMS": SUFFIX_NOT_ACRONYMS,
        "CONJUNCTIONS": CONJUNCTIONS,
        "PREFIXES": PREFIXES,
        "CAPITALIZATION_EXCEPTIONS": CAPITALIZATION_EXCEPTIONS,
        "REGEXES": REGEXES,
    },
}

ERRORS = {
    "invalid": "Invalid setting '{0}'. Refer to {1} for available settings.",
    "removed": "'{0}' has been removed. Refer to '{1}' for available settings.",
}

SETTINGS_DOC = "https://github.com/jcp/django-names/blob/master/README.md#settings"


class Settings:
    """
    Constructs a `Settings` object to handle improperly configured settings,
    combine default and user-defined settings, and expose settings as properties.

    Example:

      >>> from names.settings import settings
      >>> print(settings.OPTIONS)
    """

    def __init__(self):
        self.defaults = DEFAULTS
        self.user_settings = self._get_user_settings()
        self._setup()

    def _get_user_settings(self):
        """
        Retrieves user-defined settings and handles improperly configured settings.
        """
        settings = getattr(dj_settings, "NAMES_SETTINGS", {})

        for setting in settings:
            if setting not in self.defaults:
                raise AttributeError(ERRORS["invalid"].format(setting, SETTINGS_DOC))

        if "OPTIONS" in settings:
            for const in settings["OPTIONS"]:
                if const not in self.defaults["OPTIONS"]:
                    raise AttributeError(ERRORS["invalid"].format(const, SETTINGS_DOC))

        return settings

    def _setup(self):
        """
        Exposes settings as properties and combines defaults and user-defined settings.
        """
        for setting, default in self.defaults.items():
            setattr(self, setting, self.user_settings.get(setting, default))

        # Add default `OPTIONS` that were omitted from user-defined settings.
        for setting, val in self.defaults["OPTIONS"].items():
            if setting not in self.OPTIONS:
                self.OPTIONS.update({setting: val})


settings = Settings()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import shutil
import sys

from io import open
from setuptools import find_packages, setup


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    shutil.rmtree("dist")
    shutil.rmtree("build")
    shutil.rmtree("django_names.egg-info")
    sys.exit()

here = os.path.abspath(os.path.dirname(__file__))

install_requirements = ["Django>=1.11", "nameparser>=1.0.4"]

about = {}
with open(os.path.join(here, "names/__init__.py"), "r", encoding="utf-8") as f:
    exec(f.read(), about)

with open("README.md", "r", encoding="utf-8") as f:
    readme = f.read()

setup(
    name=about["__package_name__"],
    author=about["__author__"],
    author_email=about["__author_email__"],
    description=about["__description__"],
    include_package_data=True,
    install_requires=install_requirements,
    license=about["__license__"],
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["tests*"]),
    python_requires=">=3.6",
    url=about["__url__"],
    version=about["__version__"],
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)
